#include "version.h"
#include "common.h"
#include "readmap.h"
#include "char.h"
#include "map_util.h"
#include "lssproto_serv.h"
#include "net.h"
#include "anim_tbl.h"

/*#define MAPEDITORTROUBLE*/
#define SPR_kmydamY CG_HIT_MARK_10 // anim_tbl.h に登録されるまで毒のエフェクト

/*------------------------------------------------------------
 * 歩けるかどうか判断する。キャラが飛んでいるかどうか引数がある。
 * 引数
 *  ff          int     floor
 *  fx          int     x座標
 *  fy          int     y座標
 *  isfly       int     飛んでいるかどうか
 * 返り値
 *  歩ける      TRUE(1)
 *  歩けない    FALSE(0)
 ------------------------------------------------------------*/
BOOL MAP_walkAbleFromPoint( int ff, int fx, int fy, BOOL isfly )
{
    int map[2];

    if( !MAP_getTileAndObjData( ff,fx,fy, &map[0], &map[1] ) ){
        return FALSE;
	}

    if( isfly ){
        int i;
        for( i = 0 ; i < 2 ; i ++ )
            if( MAP_getImageInt( map[i], MAP_HAVEHEIGHT ) == TRUE  ){
                return FALSE;
						}
        return TRUE;
    }else{
        switch( MAP_getImageInt( map[1], MAP_WALKABLE ) ){
        case 0:
            return FALSE;
            break;
        case 1:
            if( MAP_getImageInt( map[0], MAP_WALKABLE ) == 1 ){
                return TRUE;
			}else{
                return FALSE;
			}
            break;
        case 2:
            return TRUE;
            break;
        default:
            return FALSE;
            break;
        }
    }
    return FALSE;
}



/*------------------------------------------------------------
 * 歩けるかどうか判断する。
 * 引数
 *  index       int     Char配列での Index
 *  ff          int     floor id
 *  fx          int     x座標
 *  fy          int     y座標
 * 返り値
 *  歩いてよい      TRUE(1)
 *  歩いてはダメ    FALSE(0)
 ------------------------------------------------------------*/
BOOL MAP_walkAble( int index,int ff, int fx, int fy)
{
    /*   透明キャラならOK    */
    if( CHAR_getFlg( index, CHAR_ISTRANSPARENT ) )      return TRUE;

    // Arminius 7.9 Airplane ok
    if( CHAR_getInt( index, CHAR_WHICHTYPE ) == CHAR_TYPEBUS) return TRUE;
    if( CHAR_getWorkInt( index, CHAR_WORKPARTYMODE) == CHAR_PARTY_CLIENT ) {
      int oyaindex = CHAR_getWorkInt( index, CHAR_WORKPARTYINDEX1);
      if( CHAR_CHECKINDEX( oyaindex)) {
        if (CHAR_getInt( oyaindex, CHAR_WHICHTYPE ) == CHAR_TYPEBUS) return TRUE;
      }
    }

    return MAP_walkAbleFromPoint( ff,fx,fy,CHAR_getFlg(index,CHAR_ISFLYING)|CHAR_getWorkInt(index,CHAR_WORKSKYWALKER) );
}

/*------------------------------------------------------------
 * 座標を指定して、そこのオブジェクトに高さがあるかどうか
 * 引数
 *  fl              int     フロア
 *  x               int     x 座標
 *  y               int     y 座標
 * 返り値
 *  高さがある  TRUE
 *  高さがない  FALSE
 ------------------------------------------------------------*/
BOOL MAP_haveHeight( int fl, int x, int y )
{
    int     map[2];

    /*   マップデータを得てくる  */
    if( !MAP_getTileAndObjData( fl,x,y, &map[0], &map[1] ) )
        return FALSE;

    return MAP_getImageInt( map[1], MAP_HAVEHEIGHT );
}

/*------------------------------------------------------------
 *
 ------------------------------------------------------------*/
static void MAP_dealprepostevent( int index, BOOL mode )
{
    int     map[2];
    int     i;
    int     damaged=FALSE;

    if( CHAR_getFlg(index,CHAR_ISFLYING) )
        /*    氏匹月支勾卞反窒手仄卅中  */
        return;

    if( !MAP_getMapDataFromCharIndex( index  ,  map ) ) return;

    for( i = 0 ; i < 2 ; i ++ ){
        /*  damage */
        int damage;
        damage = MAP_getImageInt(map[i],
                                 mode ? MAP_INTODAMAGE : MAP_OUTOFDAMAGE);
        if( damage != 0 ){
            /*  母丢□斥汹毛恳匹  蜇允月及匹  寞  木赘尹    */
            int opt[2]={SPR_kmydamY,-damage};
            damaged=TRUE;
            CHAR_setInt(index, CHAR_HP,
                        CHAR_getInt(index,CHAR_HP) + damage );
            CHAR_complianceParameter(index);
            /*  母丢□斥熬仃凶巨白尼弁玄请允    */
            CHAR_sendWatchEvent(CHAR_getWorkInt(index,CHAR_WORKOBJINDEX),
                                CHAR_ACTDAMAGE,opt,2,TRUE);
        }
    }
    if( damaged )
    	CHAR_sendStatusString(index,"M");
}


/*----------------------------------------
 * 公及正奶伙卞昙月蟆卞裟壬木月［仇仇匹公仇井日转仁午｝xxx烟及
 * 仪毛  蜇允月［
 * 娄醒
 *  index       int      Char配列での Index
 *  flooor      int      floor id
 *  fx          int     x座標
 *  fy          int     y座標

 * 忒曰袄
 *  卅仄
 ----------------------------------------*/
void MAP_preovered( int index )
{
    MAP_dealprepostevent( index, FALSE );
}

/*----------------------------------------
 * 公及正奶伙卞昙匀凶  卞裟壬木月［
 * 娄醒
 *  index       int     Char配列での Index
 * 忒曰袄
 *  卅仄
 ----------------------------------------*/
void MAP_postovered( int index )
{
    MAP_dealprepostevent( index, TRUE );
}


/*----------------------------------------
 * 平乓仿及璃曰及穴永皿毛允屯化霜月
 * 娄醒
 *  fd          int
 *  charaindex  int     平乓仿及奶件犯永弁旦
 * 忒曰袄
 *  岳      TRUE(1)
 *  撩      FALSE(0)
 ----------------------------------------*/
BOOL MAP_sendArroundCharNeedFD( int fd,int charaindex )
{
    char*   stringdata;
    int     x=CHAR_getInt(charaindex,CHAR_X);
    int     y=CHAR_getInt(charaindex,CHAR_Y);
    int     fl=CHAR_getInt(charaindex,CHAR_FLOOR);
    int     size=MAP_CHAR_DEFAULTSEESIZ;
    RECT    seekr,retr;
    seekr.x = x - (int)(size/2);
    seekr.y = y - (int)(size/2);
    seekr.width  = size;
    seekr.height = size;
#if 1
{
	int		tilesum, objsum, eventsum;
    stringdata = MAP_getChecksumFromRECT(fl,&seekr,&retr, &tilesum,&objsum,&eventsum);
    if( stringdata == NULL )
        return FALSE;

    lssproto_MC_send(fd,fl,
                    retr.x,              retr.y,
                    retr.x + retr.width, retr.y + retr.height,
                    tilesum,
                    objsum,
                    eventsum,
                    stringdata );
}
#else
    stringdata = MAP_getdataFromRECT(fl,&seekr,&retr);
    if( stringdata == NULL )
        return FALSE;

    lssproto_M_send(fd,fl,
                    retr.x,              retr.y,
                    retr.x + retr.width, retr.y + retr.height,
                    stringdata );
#endif
    return TRUE;
}


/*----------------------------------------
 * 穴永皿犯□正毛  月［
 * 娄醒
 *  charaindex      int     平乓仿及奶件犯永弁旦
 * 忒曰袄
 *  岳      TRUE(1)
 *  撩      FALSE(0)
 ----------------------------------------*/
BOOL MAP_sendArroundChar(int charaindex)
{
    int fd;
    fd = getfdFromCharaIndex( charaindex );
    if( fd == -1 )return FALSE;

    return MAP_sendArroundCharNeedFD(fd, charaindex);
}

#ifdef _CAX_LNS_NLGSUOXU 
static int YCXX[] = {39,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,43,43,43,43,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,90,90,90,90,90,90,91,91,91,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,93,93,93,93,93,93,93,94,94,94,94,94,94,94,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,99,99,99,99,99,99,100,100,100,100,100,100,100,100,100,100,101,101,101,101,101,101,102,102,102,102,102,103,103,103,103,103,104,104,104,104,104,105,105,105,105,105,105,105,105,105,106,106,106,106,106,106,106,106,106,106,106,106,106,106,106,106,107,107,107,107
};

static int YCXY[] = {39,39,40,64,65,66,76,77,78,40,41,63,66,67,73,74,75,79,82,83,40,61,62,68,70,71,72,80,81,83,39,60,69,83,40,60,83,41,47,59,63,64,65,66,67,68,69,70,71,72,78,82,40,47,48,60,63,73,76,77,78,81,39,40,44,45,46,49,60,63,74,76,78,82,40,45,46,47,48,50,60,63,75,78,83,40,48,51,55,59,60,62,78,84,40,49,50,52,55,56,57,58,61,79,85,40,50,52,54,60,72,73,75,80,81,86,40,48,49,50,51,52,53,60,72,76,77,82,87,40,47,48,49,50,51,52,59,72,78,83,88,89,90,40,52,58,72,78,84,91,92,93,40,52,57,60,61,62,63,64,65,66,67,68,69,70,71,72,74,75,76,79,85,94,95,96,40,53,57,60,73,77,80,86,97,98,99,108,109,110,40,54,57,59,72,77,78,81,87,91,92,100,107,108,109,110,111,40,54,57,59,71,77,79,82,83,88,89,90,93,98,99,101,102,103,104,105,106,107,111,112,113,39,54,57,58,59,60,61,62,63,64,65,66,70,76,77,80,84,94,95,96,97,100,101,106,112,114,38,53,56,57,60,67,71,75,77,79,85,86,93,94,99,100,101,102,106,112,114,37,53,55,56,57,61,68,72,73,74,75,76,77,78,87,88,91,92,95,96,97,98,101,103,106,112,115,120,34,35,36,53,54,62,69,73,77,89,90,102,103,106,113,116,119,121,34,52,53,54,63,68,72,77,89,90,102,103,106,113,117,118,120,34,39,48,49,50,51,52,53,63,68,69,70,71,77,90,91,102,104,105,106,113,118,119,34,39,40,41,46,47,48,49,50,51,63,67,68,77,90,92,93,94,95,96,102,104,105,106,107,112,113,118,119,34,39,42,45,46,51,63,66,72,73,74,77,91,92,93,96,99,100,101,102,103,104,107,112,114,118,34,39,43,44,45,51,64,70,71,75,77,93,95,98,102,107,108,109,110,111,112,115,118,35,39,45,51,52,53,65,69,75,77,93,94,97,103,112,115,119,121,36,39,45,53,54,66,67,68,75,77,92,93,96,97,98,99,100,101,104,112,115,119,121,36,37,38,39,40,45,54,67,75,77,91,93,95,96,100,105,108,109,110,111,112,113,114,115,116,117,118,120,121,40,45,55,75,78,90,93,94,96,99,106,108,114,115,119,120,121,41,45,56,57,58,75,79,90,93,96,98,107,114,115,119,44,55,56,57,59,76,80,85,86,87,89,92,93,96,98,111,112,113,114,115,118,41,42,43,54,57,60,61,77,81,84,87,89,91,93,96,97,111,115,117,118,119,120,121,41,42,43,54,58,59,60,61,62,63,64,78,81,83,87,88,90,93,96,111,116,117,120,41,44,52,53,54,64,65,79,80,83,86,87,89,93,95,96,112,117,45,52,64,66,79,80,83,84,85,86,87,88,89,90,91,92,93,94,96,113,114,117,45,49,50,51,64,66,69,70,71,72,75,76,78,79,80,81,84,86,88,93,96,113,118,45,49,64,66,68,72,73,74,77,78,81,82,83,84,86,88,89,90,91,92,96,111,112,119,45,48,64,66,68,79,81,83,85,86,87,88,91,93,96,105,106,107,108,109,110,111,119,45,46,47,64,65,66,67,79,80,81,82,87,90,94,96,102,103,104,109,119,64,65,78,81,82,87,90,94,96,98,99,100,101,102,105,109,118,64,65,78,79,80,81,82,87,90,94,95,97,100,106,108,109,110,111,112,113,114,115,117,63,64,65,81,87,90,93,95,96,99,106,108,115,116,62,63,64,65,66,81,87,89,90,91,92,95,97,98,106,107,108,116,62,65,66,81,86,87,89,91,96,97,116,61,66,67,81,85,87,89,92,97,98,116,62,64,67,68,81,84,87,88,93,97,98,99,116,62,63,65,68,69,81,83,86,88,94,97,98,99,63,66,70,81,82,86,87,93,97,98,99,63,64,67,80,81,85,93,97,98,99,64,65,68,69,80,81,84,85,93,97,98,99,64,65,69,80,81,82,83,84,94,95,96,97,98,99,69,80,81,82,84,97,98,70,80,81,82,84,96,98,70,71,72,73,74,75,76,80,81,82,84,96,98,73,74,75,76,77,80,82,84,95,97,98,73,80,82,84,94,96,73,80,82,83,95,98,71,72,80,82,83,91,92,94,96,97,70,81,82,90,93,96,69,81,83,89,95,69,83,84,88,94,69,85,86,87,94,70,80,81,82,83,93,94,95,96,71,72,73,74,75,76,77,78,79,80,84,89,90,91,92,94,85,86,87,88
};

static int JJXX[] = {54,55,55,56,56,56,56,57,57,57,58,58,58,58,58,58,58,58,59,59,59,60,60,60,60,60,60,60,61,61,61,61,61,61,62,62,62,62,62,63,63,63,63,63,63,64,64,64,64,64,65,65,65,65,65,65,65,65,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,74,74,74,74,74,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,77,77,77,77,77,77,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,91,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,93,93,94,94,94,94,94,94,94,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,98,98,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,99,99,99,100,100,100,100,100,100,100,100,100,100,100,100,100,101,101,101,101,101,101,101,101,101,101,102,102,102,102,102,102,102,102,102,102,102,102,103,103,103,103,103,103,103,103,103,103,104,104,104,104,104,104,104,104,104,104,104,104,104,104,105,105,105,105,105,105,105,105,105,105,105,106,106,106,106,106,106,106,106,106,107,107,107,107,107,107,107,108,108,108,108,109,109,109,109,109,110,110,111,111,111,111,111,111,111,112,112,113,113,113,114,114,115
};

static int JJXY[] = {107,106,107,98,99,100,106,99,101,105,100,101,102,103,104,115,116,117,103,114,118,102,104,112,113,114,119,123,101,104,112,120,122,124,99,100,103,112,121,96,97,98,103,112,118,96,103,112,117,119,95,103,111,116,117,118,119,120,95,104,110,115,124,95,104,108,109,110,111,112,113,114,123,95,105,106,107,108,109,110,122,90,91,92,94,121,89,93,113,114,115,116,117,118,119,120,88,92,94,112,89,90,92,94,102,103,104,105,106,107,111,91,92,94,101,107,111,94,100,101,107,111,56,57,58,94,99,102,103,107,110,56,59,94,103,107,110,56,60,94,103,107,110,56,60,94,103,107,110,57,60,95,96,97,98,99,100,101,102,103,107,110,58,61,93,94,95,96,107,111,58,61,92,96,97,98,108,111,59,61,78,79,92,99,109,111,60,62,77,80,92,100,109,111,61,62,75,76,81,82,93,100,110,62,63,74,83,94,100,110,63,64,73,84,95,99,111,63,65,71,72,85,86,96,99,112,63,66,67,68,70,87,88,89,90,91,92,95,98,113,62,69,89,90,92,93,94,95,96,97,114,62,86,87,88,115,62,87,93,94,95,96,97,116,62,66,88,93,94,95,96,116,62,65,67,68,88,93,94,95,116,62,65,69,88,89,92,115,62,65,70,90,91,115,62,65,70,90,91,92,104,105,106,107,108,114,63,64,65,66,70,87,88,89,93,103,108,113,66,67,70,87,88,89,94,95,101,102,106,107,108,113,67,68,70,85,87,88,89,90,96,100,106,113,68,70,84,86,91,97,98,99,100,101,102,106,114,70,83,87,88,92,98,103,104,106,114,70,74,75,84,89,90,91,93,97,105,106,114,70,73,76,84,92,94,96,105,107,114,70,73,77,85,93,94,95,104,108,109,110,111,112,114,69,73,78,79,84,86,96,97,104,113,114,69,73,80,82,83,85,86,98,104,68,73,80,81,86,99,103,68,73,100,103,68,72,101,102,103,68,71,64,65,66,67,68,69,70,66,67,66,67,68,66,67,66
};

static int KLXX[] = {34,35,36,36,36,36,36,37,37,37,37,37,37,38,38,38,38,38,38,38,38,38,38,38,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,40,40,40,40,40,40,41,41,41,41,41,42,43,44,45,46,47,48,49,49,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,90,90,90,90,90,90,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,98,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,100,100,100,100,100,101,101,101,101,101,101,102,102,102,102,102,103,103,103,103,103,104,104,105,105,106,106,106
};

static int KLXY[] = {62,62,62,71,72,73,74,61,62,69,70,75,77,60,61,63,64,65,66,68,75,76,77,78,59,60,63,64,65,66,67,68,69,70,71,72,73,74,78,59,62,63,64,65,66,60,61,62,63,64,61,61,61,61,61,61,61,60,61,59,60,61,58,59,61,66,67,68,92,93,105,109,110,111,112,47,48,57,62,65,66,69,70,71,72,93,94,95,101,102,103,104,105,106,107,109,112,113,48,49,50,51,52,53,54,55,56,57,59,60,61,62,63,64,65,66,67,68,69,70,71,72,93,95,96,102,103,104,107,109,110,114,37,48,49,58,63,65,70,93,96,97,103,105,106,107,108,109,110,111,112,114,37,38,47,48,58,59,62,64,65,70,71,93,96,106,110,112,114,37,62,63,70,71,93,96,107,110,113,114,37,38,70,71,93,94,95,106,107,108,109,110,111,112,114,38,70,71,92,93,106,107,109,112,114,37,38,39,71,93,94,105,106,107,109,113,33,34,35,36,37,39,40,41,69,70,72,93,94,105,106,110,111,112,35,37,40,41,42,43,54,55,68,73,74,93,94,105,106,34,37,38,42,43,44,45,46,54,55,68,74,75,93,94,105,35,37,46,55,68,76,93,94,105,36,37,38,47,48,49,50,51,52,53,54,55,56,68,77,78,79,92,93,94,105,37,48,54,57,68,79,92,94,105,37,38,48,57,58,68,79,90,91,92,93,94,95,104,105,38,39,48,58,59,68,69,73,79,88,89,91,92,94,96,104,105,39,48,58,59,60,61,62,63,64,65,66,67,69,70,72,74,79,88,91,92,93,94,95,97,103,104,105,106,39,40,41,42,48,58,59,60,61,62,63,64,65,66,67,68,69,70,71,74,78,79,89,91,92,94,95,98,100,101,102,106,48,58,59,60,71,72,73,74,75,76,77,78,87,88,89,90,92,94,96,99,106,48,58,59,71,72,73,74,75,77,78,79,80,81,82,83,84,85,86,87,92,94,96,107,48,58,59,72,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,108,109,110,111,47,58,85,86,87,88,92,95,96,110,48,58,84,85,89,90,91,97,109,48,53,58,85,86,97,108,49,53,58,85,86,87,88,96,97,98,107,50,53,58,84,87,95,96,99,100,107,50,52,53,58,84,86,95,96,101,107,50,52,53,57,58,84,86,95,96,97,98,102,103,104,105,106,107,108,109,110,50,52,53,54,55,56,57,59,84,85,86,96,97,100,101,102,50,53,54,56,57,58,59,60,61,84,85,97,98,99,102,49,56,61,62,63,84,85,97,98,101,49,56,61,64,65,84,85,97,98,99,101,48,62,65,85,97,98,100,48,62,65,85,97,100,44,45,46,47,63,65,96,98,100,45,63,65,95,96,98,99,100,46,63,65,93,94,95,96,97,99,100,47,64,65,93,94,95,99,101,48,63,65,95,99,101,49,63,65,96,99,100,50,51,62,65,69,70,71,72,96,97,98,99,52,57,58,59,60,61,65,69,73,74,75,82,83,84,96,97,98,53,54,55,56,64,65,68,75,76,77,78,79,80,81,85,96,97,62,63,67,70,71,72,86,96,97,62,65,66,70,73,80,81,82,83,87,96,63,64,67,68,69,70,74,75,76,77,78,79,84,85,88,63,69,70,74,75,76,77,79,80,81,82,86,88,62,63,69,70,71,72,74,87,88,62,69,70,86,88,61,62,69,70,86,87,61,62,69,85,87,61,69,84,85,86,61,69,69,70,70,71,72
};

static int SCXX[] = {27,27,27,27,27,28,28,28,29,29,29,29,29,29,29,29,29,29,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,31,31,32,32,33,33,33,33,33,33,33,34,34,34,34,34,34,34,34,34,34,34,34,35,35,35,35,35,35,36,36,36,36,36,36,36,36,36,36,36,36,37,37,37,37,37,37,37,37,38,38,38,38,38,38,38,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,47,47,47,47,47,48,48,48,48,48,48,48,49,49,49,49,49,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,67,67,67,67,67,67,67,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,90,90,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,94,94,94,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,99,99,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,101,101,101,101,101,101,101,101,101,101,101,101,101,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,103,103,103,103,103,103,103,103,103,104,104,104,104,104,104,104,104,104,105,105,105,105,105,105,105,105,105,105,105,106,106,106,106,106,106,106,106,106,106,106,106,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,110,110,110,110,110,110,110,110,111,111,111,111,111,111,111,111,112,112,112,112,112,112,112,112,112,113,113,113,113,113,113,113,113,113,114,114,114,114,114,114,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,117,117,117,117,117,117,118,118,118,118,118,118,118,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,120,120,120,120,120,120,120,120,120,120,120,120,120,120
};

static int SCXY[] = {67,68,69,70,71,65,66,72,64,73,78,79,80,81,82,83,84,85,53,54,55,56,57,58,59,60,61,62,63,74,75,76,77,85,52,86,51,87,51,60,61,73,74,75,88,51,59,61,74,75,76,77,78,79,80,81,89,51,58,62,75,82,90,47,48,49,50,51,57,62,75,83,86,87,90,49,56,62,76,83,85,86,90,49,55,62,76,83,85,90,49,54,61,76,82,85,90,49,54,61,76,81,85,86,90,48,49,54,61,73,74,75,76,77,78,79,80,81,85,87,90,48,54,62,73,76,85,88,90,48,54,62,72,76,85,89,90,48,54,55,56,63,66,67,68,69,70,71,76,77,85,90,91,48,57,64,65,66,78,86,90,91,48,58,59,79,87,90,91,48,60,80,88,91,48,61,62,63,64,81,89,48,65,82,90,101,48,66,83,91,100,102,110,48,66,84,92,93,99,103,108,109,111,48,66,84,85,94,98,103,107,112,48,66,83,90,95,96,97,103,106,113,47,66,82,89,103,105,113,119,120,121,47,66,82,88,104,114,118,122,47,66,82,87,90,91,92,93,94,95,96,97,98,99,100,115,116,117,123,124,47,66,82,86,91,92,101,102,103,104,105,106,107,108,124,46,65,83,84,85,93,101,102,109,124,45,64,82,83,84,94,100,103,110,111,112,113,114,115,116,117,118,119,120,124,44,63,81,82,85,94,99,104,112,113,114,115,116,117,118,124,43,62,83,86,87,88,89,90,91,92,93,94,95,96,97,98,104,105,106,107,108,109,112,124,42,55,56,57,58,59,60,61,84,85,94,104,110,111,124,41,54,76,77,78,79,80,81,86,94,124,40,52,53,75,82,87,94,125,39,46,47,48,49,50,51,75,83,88,94,126,39,46,75,84,89,94,127,39,45,75,84,89,94,128,39,45,74,84,89,94,128,39,45,49,50,51,52,53,54,55,56,74,84,90,94,127,38,44,50,57,73,85,86,90,94,126,37,44,51,58,72,87,90,94,95,125,36,43,51,59,72,88,91,96,125,35,43,52,60,72,89,92,97,125,35,43,53,61,62,63,72,89,93,94,98,125,35,38,43,52,53,63,64,72,74,75,76,77,78,89,94,99,125,35,38,43,51,65,66,71,73,79,89,95,96,100,125,35,38,39,43,50,67,71,72,80,89,97,98,101,102,103,104,125,35,38,39,43,49,68,70,72,80,90,99,105,125,35,38,39,43,49,68,69,72,81,91,100,106,125,35,38,40,43,49,68,69,72,81,92,101,106,124,35,38,41,42,49,68,69,72,81,93,98,99,100,101,102,106,124,34,38,39,49,68,72,73,81,93,96,97,106,123,32,33,39,49,68,73,81,93,94,95,106,120,121,122,31,39,48,49,67,73,81,92,93,107,120,31,38,47,59,60,61,62,63,64,65,66,74,81,90,91,92,97,98,99,107,120,31,32,37,38,39,40,41,42,43,44,45,46,50,51,52,53,54,55,56,57,59,66,75,76,77,78,79,80,81,82,88,89,90,91,96,100,108,120,32,49,50,58,59,65,79,83,84,95,101,102,103,104,105,106,107,109,120,33,43,44,45,46,47,48,62,63,64,78,85,86,87,88,89,90,91,92,93,94,108,110,119,34,35,43,61,62,77,109,111,112,113,118,35,43,60,76,110,113,118,34,42,59,76,111,113,118,34,41,55,56,58,76,111,112,118,35,40,43,44,45,46,47,48,54,57,76,83,84,85,111,118,35,39,41,42,49,50,54,76,82,86,117,34,38,39,40,51,54,76,80,81,87,116,34,52,54,76,77,78,79,87,116,33,53,54,76,87,116,33,53,54,76,87,116,33,53,54,75,76,87,116,117,127,128,129,33,53,54,55,74,77,88,117,118,119,120,121,122,123,126,129,32,54,55,64,65,73,77,89,90,91,124,125,129,32,53,54,65,72,77,92,93,94,95,96,97,98,99,100,101,102,124,125,126,127,128,31,53,54,66,71,77,78,103,124,31,53,54,66,67,70,78,104,123,30,37,51,52,53,65,68,69,78,105,123,29,36,38,49,50,52,65,71,72,78,105,123,28,34,35,38,44,45,46,47,48,49,50,51,52,63,64,65,71,72,78,105,111,112,113,114,115,116,117,118,119,120,121,122,123,28,31,32,33,39,44,49,52,61,62,63,71,72,78,105,111,29,30,31,39,44,49,61,72,73,74,75,76,77,78,104,105,111,29,40,44,49,61,73,103,111,29,40,44,49,61,74,103,111,28,29,40,44,50,61,75,102,110,29,41,42,43,50,61,75,102,109,28,49,62,75,102,108,28,49,62,75,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,107,29,43,44,45,46,47,48,62,64,65,66,67,68,69,70,71,72,73,74,75,86,107,29,42,62,64,86,107,29,30,41,62,64,87,107,31,32,33,34,35,40,62,64,65,66,67,68,69,70,87,107,32,33,34,35,36,37,38,39,40,58,62,70,88,107
};

static int FCXX[] = {28,29,29,29,30,30,31,31,31,31,31,32,32,32,32,33,33,33,33,33,33,33,33,34,34,34,34,34,34,34,34,34,35,35,35,35,35,35,35,35,35,35,36,36,36,36,37,37,37,37,37,38,38,38,38,38,38,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,52,52,52,52,52,52,52,53,53,54,54,54,55,55,55,55,55,56,56
};

static int FCXY[] = {33,33,34,37,35,36,35,51,52,53,54,35,36,50,54,34,36,37,38,39,40,50,54,33,34,40,41,42,43,50,53,54,33,34,43,44,48,49,50,51,52,53,33,34,51,52,33,34,52,53,54,34,35,52,53,55,56,34,35,51,52,53,54,56,27,28,29,32,33,36,51,54,56,58,59,29,30,31,37,51,52,54,56,59,29,32,33,34,35,36,38,47,48,49,50,51,52,53,54,56,59,60,61,62,29,32,36,38,44,45,46,47,49,50,51,53,56,57,58,59,60,61,23,24,25,28,29,31,36,38,43,45,46,47,48,49,50,51,53,54,55,56,59,61,22,26,27,28,29,30,32,36,39,40,41,42,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,20,21,22,23,26,27,31,32,33,34,35,36,39,40,41,42,43,44,47,48,49,51,55,56,57,60,23,27,32,35,36,37,38,42,47,49,50,51,55,56,57,60,23,26,31,32,33,34,35,40,41,48,49,50,51,59,60,23,25,29,30,31,36,37,38,39,59,23,25,26,27,28,29,30,31,32,33,34,35,36,37,23,24,25,29,33,34,35,23,26,28,29,30,31,32,23,26,22,25,26,22,23,24,25,26,25,26
};

static int QGGXX[] = {42,42,42,42,42,42,42,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,63,63,63,64,64,65,65,65,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,69,69
};

static int QGGXY[] = {52,53,54,55,56,58,59,52,55,57,58,59,51,52,53,54,55,56,57,59,61,62,50,51,58,59,60,62,49,50,58,59,62,48,49,59,61,64,65,66,67,68,71,72,73,74,75,83,49,50,59,61,64,69,70,76,78,79,80,81,82,43,44,45,46,47,49,51,59,62,63,68,69,70,71,72,73,74,75,76,77,80,43,48,51,59,66,67,68,70,71,72,73,74,75,76,77,78,79,43,50,52,58,59,60,61,63,64,65,69,79,43,49,50,52,53,54,55,56,57,58,61,62,68,79,80,42,51,52,55,56,58,67,68,79,80,81,42,51,52,53,54,56,57,58,67,79,81,42,50,52,54,55,57,58,59,67,79,80,81,43,44,49,50,51,53,56,59,67,79,45,48,51,52,53,54,55,58,66,79,42,43,44,47,48,51,52,58,66,68,74,75,76,77,78,42,43,44,45,46,48,50,51,58,66,68,69,72,73,42,44,45,46,47,48,49,50,58,66,67,70,72,42,45,46,49,50,58,66,70,72,43,44,47,48,50,58,71,48,51,58,52,58,52,57,58,52,55,56,57,58,59,60,50,51,52,53,54,55,59,60,61,62,50,54,55,56,57,58,61,54,57
};

static int DNXX[] = {22,23,23,23,24,24,25,25,26,26,26,26,27,27,27,27,27,28,28,28,28,29,29,29,29,29,29,30,30,30,30,30,30,30,30,30,30,30,30,31,31,31,31,31,32,32,32,32,32,33,33,33,33,33,33,33,33,33,34,34,34,34,34,34,34,34,35,35,35,35,35,35,35,35,35,36,36,36,36,36,36,36,36,36,36,36,36,36,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,38,38,38,38,38,38,38,38,38,38,38,38,38,38,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,42,43,43,43,43,43,44,44,44,44,44,45,45,45,45,45,45,46,46,46,47,47
};

static int DNXY[] = {33,32,33,34,31,35,31,36,31,32,36,37,31,33,34,35,37,31,33,34,37,30,33,34,35,36,39,26,27,28,29,30,34,35,36,37,38,39,40,31,37,40,41,42,32,37,39,40,42,28,31,36,37,38,39,40,41,43,28,29,30,35,36,40,42,44,31,34,35,36,40,42,44,47,48,30,32,34,35,36,40,41,42,43,44,45,46,48,29,31,32,33,34,36,37,38,39,40,41,42,43,44,45,46,47,29,33,34,35,36,37,39,40,41,42,44,45,46,48,29,30,31,32,33,34,37,38,39,41,43,44,45,46,47,48,49,51,52,31,32,34,37,38,39,40,43,44,45,46,49,50,51,32,33,34,37,38,39,40,41,42,43,44,45,46,48,49,50,51,32,33,37,38,39,41,44,48,49,50,51,37,38,39,41,51,36,38,39,40,41,36,37,38,39,40,41,37,38,40,38,39
};

static int KAXX[] = {32,33,34,34,34,34,34,34,35,35,35,35,35,35,35,35,35,35,35,36,36,36,36,36,37,37,37,37,38,38,38,38,38,38,38,38,38,38,38,39,39,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,91,91,91,91,91,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,100,100,100,100,101,101,101
};

static int KAXY[] = {65,65,60,61,62,63,64,66,37,38,39,40,41,42,43,44,45,59,67,36,46,58,68,69,35,47,58,70,35,48,58,60,61,62,63,64,65,66,71,35,48,58,60,67,72,99,100,101,35,48,58,60,68,73,98,101,35,48,58,60,61,69,74,97,102,34,49,58,60,61,62,67,70,74,96,97,102,34,50,51,57,60,61,63,66,68,71,75,89,90,91,92,93,94,95,96,97,102,33,48,52,53,54,55,56,60,61,64,65,69,72,76,90,96,98,99,100,101,32,38,39,40,41,42,43,46,47,49,59,61,64,65,70,73,76,82,83,91,96,33,34,37,44,45,50,53,54,55,56,57,58,61,64,71,74,77,78,79,80,81,84,88,89,90,91,96,35,36,50,52,61,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,85,87,91,92,97,104,105,106,107,108,38,39,40,41,50,52,61,64,73,76,79,86,93,98,103,108,37,39,42,50,51,61,65,73,77,78,94,99,101,102,107,37,39,40,41,42,43,44,45,46,47,50,52,61,66,73,77,78,81,94,100,102,106,116,37,40,43,48,50,52,55,56,57,61,65,73,76,79,81,95,101,103,105,114,115,116,37,40,43,49,50,52,53,55,58,61,64,74,75,80,81,95,100,104,112,113,115,38,40,44,49,51,52,55,58,61,63,74,75,76,81,96,100,111,115,38,40,44,48,49,51,55,59,61,62,73,77,81,96,100,104,105,106,107,108,109,110,115,116,37,40,44,48,49,51,52,53,54,55,59,61,62,73,78,81,96,100,103,117,36,40,41,44,48,49,59,60,63,69,72,77,81,96,100,102,116,37,40,45,47,50,58,60,62,68,69,73,76,81,96,99,101,110,116,38,39,40,41,42,43,44,45,46,51,52,53,54,55,56,57,59,62,67,70,74,75,80,81,95,98,100,110,115,40,45,58,62,66,69,74,75,79,90,91,92,93,94,98,100,110,111,114,36,40,41,42,43,44,45,46,47,57,62,65,68,74,76,77,78,90,98,100,110,111,114,36,37,44,47,48,49,50,51,52,53,54,55,56,61,64,67,74,76,78,90,99,101,111,113,36,38,42,43,46,47,53,54,55,56,57,58,59,60,61,63,66,73,76,79,90,99,101,102,111,112,113,36,39,40,41,46,47,52,62,67,69,70,71,72,77,80,81,82,90,100,101,102,112,113,37,41,45,47,51,60,61,68,77,82,83,84,85,86,87,88,89,90,101,102,112,38,45,48,50,59,68,69,70,71,72,73,74,75,76,77,78,88,89,90,91,92,93,94,95,96,97,102,103,113,39,40,44,48,50,58,63,67,70,78,79,80,88,97,102,104,105,106,107,108,109,110,114,37,40,41,43,44,47,50,56,57,58,62,64,65,66,70,79,80,81,88,97,98,102,110,114,37,38,39,41,42,43,44,47,50,51,52,53,54,55,56,61,64,65,70,80,81,82,88,99,102,110,114,37,38,39,40,44,46,47,49,50,57,58,59,60,63,66,70,81,82,83,84,85,88,90,91,100,102,110,114,40,41,42,43,44,45,47,48,49,55,56,62,67,70,81,83,85,86,87,90,91,99,102,110,114,35,41,43,45,46,49,53,54,63,67,70,80,84,85,87,90,91,97,98,99,102,107,110,114,35,37,38,39,40,41,42,46,49,54,63,67,71,79,85,86,87,89,91,94,95,96,103,106,108,109,111,114,35,36,37,39,40,41,46,49,55,63,67,72,79,84,85,86,87,88,91,93,94,103,104,105,109,112,115,46,50,56,63,67,73,75,79,83,87,91,95,96,97,98,99,100,101,109,113,116,46,51,52,57,64,68,74,75,76,79,82,88,89,90,91,101,109,112,115,117,47,53,58,65,69,74,75,77,78,79,81,84,85,86,87,88,89,90,91,96,97,98,101,106,107,108,109,110,111,112,113,114,118,119,120,47,54,59,60,65,70,74,75,80,83,90,91,95,99,101,105,109,111,113,120,47,53,54,55,56,57,61,65,71,74,75,79,82,90,95,100,101,105,109,110,112,115,116,120,47,52,54,58,62,65,71,74,75,76,77,78,81,90,94,101,105,108,110,111,112,113,114,117,121,47,51,53,59,63,65,71,73,77,80,90,92,93,96,97,98,101,105,107,110,111,117,121,46,50,51,52,60,64,65,70,72,73,77,80,90,91,92,93,96,99,105,106,109,110,111,112,113,114,115,116,117,121,46,54,55,56,57,61,65,69,71,73,77,80,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,107,108,120,46,51,52,53,58,62,68,70,72,76,80,86,90,95,96,99,100,101,105,107,108,118,119,120,46,48,49,50,59,63,67,69,71,75,81,85,90,96,98,99,101,104,106,109,117,46,47,60,63,64,65,66,68,71,74,76,77,78,79,80,81,84,89,97,99,100,102,103,104,105,109,116,60,63,67,70,72,73,74,75,76,77,81,82,83,87,88,96,99,101,102,109,116,60,64,65,66,69,73,86,96,101,108,116,61,62,63,68,74,75,86,97,100,107,116,61,62,67,75,86,98,99,100,107,116,62,67,76,86,87,89,90,91,92,93,94,95,96,97,98,107,116,63,67,76,86,87,88,107,111,112,113,114,115,63,64,67,76,85,86,87,88,107,110,64,67,76,84,87,107,109,64,65,66,75,83,88,107,109,63,66,72,73,74,80,81,82,83,84,85,86,87,88,108,109,62,65,67,68,69,70,71,79,82,108,109,62,63,64,70,72,77,78,80,81,61,62,63,70,71,73,77,79,61,62,72,73,74,75,76,77,78,61,62,73,77,74,75,76
};

static int TMXX[] = {33,33,34,34,34,34,34,34,35,35,35,35,35,35,35,35,35,35,35,35,35,36,36,36,36,36,36,36,36,36,36,37,37,37,37,37,37,37,37,37,37,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,39,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,73,73,73,73,73,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84
};

static int TMXY[] = {78,85,58,77,79,84,85,86,56,57,58,74,75,76,80,81,83,87,88,89,90,57,58,73,74,82,86,87,88,90,91,57,59,60,61,62,63,73,85,89,91,57,59,64,65,66,67,68,69,70,71,72,77,78,79,80,81,82,83,84,90,91,57,61,74,75,76,84,85,91,55,56,57,60,61,74,84,86,92,51,52,54,55,59,61,74,84,87,88,48,55,58,61,73,84,88,47,48,55,57,61,72,84,88,47,48,56,57,61,71,84,88,47,48,56,57,61,71,84,88,47,48,57,58,60,61,71,84,88,42,43,44,47,57,59,60,61,62,63,64,65,66,67,71,79,80,81,82,83,84,85,86,87,89,39,40,41,44,45,46,47,48,49,50,51,52,53,54,55,56,61,67,71,79,80,81,83,84,90,39,61,67,71,79,81,82,85,90,36,37,38,47,48,49,50,51,60,67,70,78,79,81,86,87,90,37,46,47,48,49,51,52,59,63,64,65,66,67,70,78,79,81,87,91,37,41,45,49,50,51,52,57,58,62,67,68,69,70,71,72,73,74,75,76,78,79,80,81,88,91,37,40,41,42,44,49,52,53,54,57,61,67,70,76,79,80,81,82,89,91,35,36,38,39,40,41,42,43,44,45,48,49,52,53,54,55,57,60,62,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,82,90,92,35,39,40,41,43,44,45,46,47,48,49,50,51,54,55,57,60,63,65,67,71,74,77,78,80,81,82,91,92,35,38,39,40,42,48,49,50,51,54,55,57,58,59,64,67,71,73,76,77,81,91,92,33,34,37,38,39,40,41,48,51,52,53,54,55,56,57,58,63,64,65,67,71,73,75,80,91,92,33,36,37,38,39,40,49,52,55,56,57,58,67,68,69,70,71,72,73,74,78,79,80,92,33,34,35,36,37,38,40,49,50,55,56,57,70,71,72,73,76,77,82,83,84,85,86,87,92,33,37,38,39,40,41,49,50,52,55,56,57,69,70,72,73,74,75,80,81,87,33,37,38,39,40,41,48,50,51,52,53,54,55,56,57,58,69,72,73,74,75,78,79,80,87,33,34,38,39,42,48,49,50,51,52,53,54,55,57,68,69,76,77,87,34,35,37,38,39,40,41,42,44,45,46,47,48,49,50,54,55,56,57,68,69,77,78,84,87,35,38,39,40,41,42,43,44,45,48,53,57,58,68,77,79,80,84,85,88,34,40,41,42,44,45,46,51,52,57,58,68,69,77,80,83,85,89,90,91,31,32,33,40,42,43,44,50,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,69,77,79,83,85,86,88,30,31,39,40,41,42,43,48,49,52,59,62,67,77,78,82,86,88,30,33,42,43,44,45,46,47,48,49,50,51,59,63,67,76,77,81,86,87,88,30,31,32,60,64,68,75,76,77,80,81,85,86,61,65,69,70,71,72,73,74,75,76,77,79,80,81,82,83,84,86,62,66,73,76,77,78,84,85,86,63,67,73,78,82,86,64,68,71,72,78,65,69,70,71,78,66,70,71,72,73,74,75,76,77,78,79,80,66,68,69,70,71,72,73,74,75,76,77,78,79,80,67,68,70,71,75,79,68,69,70,71,72,73,74,75,80,68,71,74,76,78,80,81,68,72,75,77,79,82,83,84,85,68,69,70,71,72,73,76,78,79,80,82,83,86,68,72,81,82,83,84,86,73,74,75,76,77,78,79,80,81,86,73,74,82,83,84,85,86
};

static int KTXX[] = {62,62,62,62,62,62,62,63,63,63,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,90,90,90,90,90,90,91,91,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,94,94,94,94,94,94,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,98,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,99,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,101,101,101,101,101,102,102,102,102,102,102,102,103,103,103,103,103,103,103,103,104,104,104,104,104,104,104,104,104,104,105,105,105,105,105,105,105,105,105,105,105,105,105,105,106,106,106,106,106,106,106,106,107,107,107,107,107,107,107,108,108,108,108,109,109,109,109,109,109,110,110,110,110,110,110,110,110
};

static int KTXY[] = {90,91,92,93,94,95,96,90,96,105,90,96,97,98,99,100,102,104,105,88,89,90,91,92,100,102,103,105,106,109,110,83,84,85,86,87,88,90,91,99,102,104,105,106,107,108,109,82,87,90,99,102,105,108,110,111,112,81,87,90,98,99,100,101,103,105,106,107,109,112,74,75,76,77,78,79,80,84,85,86,87,91,97,100,102,104,105,110,112,74,75,83,85,88,89,92,96,100,102,104,105,110,111,112,113,114,115,75,77,78,82,85,90,91,92,93,94,95,96,100,101,103,104,105,106,107,108,109,110,111,113,114,115,75,77,79,80,81,82,83,84,91,94,96,97,98,99,100,101,102,103,108,109,110,113,114,115,75,77,80,91,94,97,99,100,101,102,103,104,105,106,107,110,111,112,113,75,78,80,91,94,96,100,101,110,112,113,116,73,74,78,79,92,95,99,100,106,109,110,113,114,115,116,72,76,77,78,91,94,96,98,100,107,108,110,112,113,114,116,71,75,76,77,80,81,82,83,84,85,86,87,88,89,90,93,95,97,99,106,107,110,114,116,71,74,75,78,79,92,94,96,97,98,105,106,111,112,115,117,71,73,74,75,79,80,81,82,83,84,92,95,96,98,104,105,111,112,113,115,117,71,74,85,86,87,88,89,90,91,93,95,96,97,98,99,100,111,113,115,117,72,74,91,92,94,95,97,98,99,101,111,113,115,117,72,74,84,85,86,87,88,90,94,95,96,97,98,99,100,101,102,111,113,114,116,117,72,74,83,89,94,101,102,103,104,105,106,107,108,109,111,113,114,115,116,71,72,73,74,80,81,83,88,90,94,95,96,97,99,100,101,102,104,105,110,111,112,113,114,115,70,71,74,79,82,83,84,85,86,87,89,90,91,92,93,94,102,103,104,105,108,109,110,111,115,69,70,71,72,73,74,75,76,78,82,83,85,88,89,90,94,102,104,105,106,107,108,109,111,115,67,68,75,76,77,78,79,80,81,86,94,102,104,107,109,110,111,112,113,114,116,67,77,78,85,87,88,89,90,91,95,101,102,104,107,109,114,115,116,67,77,78,85,92,95,100,102,103,104,105,106,109,67,76,77,78,85,93,95,100,103,108,109,67,77,85,93,95,100,102,107,108,67,68,69,70,71,72,73,74,77,85,94,95,100,101,102,103,104,105,106,107,75,86,94,95,100,102,106,75,79,80,81,82,83,87,94,96,99,100,101,104,105,74,75,76,77,78,79,84,88,94,96,98,99,100,103,104,106,85,89,94,96,97,99,102,103,107,85,89,94,95,98,99,100,101,102,103,104,108,86,88,94,95,96,97,98,101,102,103,105,106,109,86,88,93,97,101,102,106,107,108,109,86,87,88,89,90,91,92,93,94,95,96,97,98,99,101,102,109,86,87,93,101,110,85,88,92,93,94,101,110,85,86,87,88,89,90,91,100,85,86,88,89,91,92,93,98,99,100,85,86,87,88,89,90,91,92,93,94,95,96,97,98,83,84,85,89,94,95,96,98,82,83,84,85,96,97,98,80,81,82,83,76,77,78,79,81,83,75,76,77,78,79,80,81,83
};


static int WLXX[] = {44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,70,70,70,70,70,70,71,71,71,71,71,71,72,72,72
};

static int WLXY[] = {63,64,65,66,40,41,42,43,44,46,47,48,52,53,54,55,59,60,61,62,67,68,39,40,44,45,49,50,51,55,56,57,58,68,38,39,44,45,46,47,48,49,50,51,57,58,59,64,65,66,68,37,38,41,42,43,45,51,55,56,60,61,64,66,67,68,37,39,40,41,42,43,44,45,52,55,61,67,68,37,43,53,55,61,67,68,69,70,72,73,74,75,76,37,44,51,52,53,54,61,67,71,77,37,44,51,52,53,55,61,67,77,36,44,46,50,51,55,56,57,58,59,60,61,66,67,68,69,70,71,72,73,74,75,77,35,44,45,47,48,49,50,51,56,57,61,62,63,66,67,76,77,35,44,45,47,51,54,55,57,61,64,65,76,35,44,45,46,47,48,49,50,51,52,53,58,60,61,62,63,64,65,76,77,35,43,44,47,50,51,53,54,55,56,58,59,60,61,63,65,76,77,36,43,44,45,46,48,49,51,52,53,55,56,57,61,62,64,66,71,72,73,74,75,76,77,36,37,42,43,44,47,49,51,53,54,57,58,61,62,65,67,70,72,73,77,37,38,39,40,41,42,43,49,50,52,53,54,55,57,62,65,68,69,71,74,75,76,77,37,38,39,40,41,43,49,53,54,55,57,62,65,66,67,69,71,39,42,43,49,50,51,52,53,54,55,57,62,64,65,69,71,39,40,41,48,51,53,55,56,57,62,65,66,69,71,40,47,48,50,51,52,53,54,56,57,58,59,60,61,62,66,67,68,71,47,49,50,51,52,54,56,57,68,69,70,45,46,47,48,49,50,51,53,55,57,43,44,45,46,48,49,52,53,54,57,58,42,43,44,45,46,47,48,49,50,51,53,54,55,56,42,45,46,49,50,43,44,45,47,48,49,43,44,46,47,48,49,43,45,49
};

static int HETXX[] = {16,16,16,16,16,16,16,16,16,16,16,16,16,16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,18,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,20,20,20,20,20,20,20,20,20,21,21,21,21,21,22,22,22,22,23,23,23,23,23,23,23,23,23,24,24,24,24,24,24,24,24,24,24,24,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,25,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,29,30,30,30,30,30,30,30,30,30,30,30,30,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,31,32,32,32,32,32,32,32,32,32,32,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,35,35,35,35,35,35,35,35,35,35,35,35,36,36,36,36,36,36,36,36,36,36,36,36,36,36,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,37,38,38,38,38,38,38,38,38,38,38,38,38,38,38,38,39,39,39,39,39,39,39,39,39,39,39,39,39,39,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,43,43,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,67,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,72,72,72,72,72,73,73,73,73,73,73,73,74,74,74,74,75,75,75,75,76,76,77,77,77,77,77,77,78,78,79,79,80,80,80,81,81,81,82,82,82,82,83,83,83,83,83,83,83,84,84,84,84,84,84,85,85,85
};

static int HETXY[] = {18,19,20,25,26,27,28,29,36,37,38,39,40,41,17,21,22,23,24,30,31,32,33,34,35,42,43,47,48,16,44,45,46,48,16,28,33,34,35,36,37,38,39,40,41,48,17,27,29,30,31,32,42,43,48,17,18,26,43,48,18,26,43,48,18,26,30,31,32,33,44,48,59,18,26,30,34,44,47,59,60,65,66,67,18,19,26,30,35,44,46,47,57,58,59,61,62,63,64,68,18,20,21,26,30,36,37,44,48,49,50,51,52,53,54,55,56,63,69,70,17,22,26,29,31,32,38,39,40,41,42,43,44,54,55,56,57,58,63,64,71,72,73,26,29,32,33,34,35,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,65,66,73,26,28,32,36,37,38,39,40,41,46,54,63,67,68,69,73,26,28,32,42,43,44,47,54,64,65,69,73,18,19,20,21,22,23,24,25,26,28,29,30,31,44,48,49,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,73,19,28,44,50,52,54,66,67,70,73,20,28,29,44,49,52,55,56,57,58,59,67,70,71,73,21,28,30,31,32,33,34,45,48,52,60,66,70,71,73,22,28,35,45,47,52,56,61,66,69,71,73,22,28,36,44,45,47,52,56,57,61,65,68,71,73,23,28,35,36,37,38,39,40,41,42,43,47,51,56,58,61,64,68,71,73,23,28,34,36,37,47,50,56,59,60,61,63,68,71,73,24,25,29,34,38,46,49,56,61,62,69,71,73,81,26,30,31,32,35,39,40,41,42,43,44,45,49,53,54,55,56,60,66,70,71,74,80,81,27,32,35,43,44,48,53,55,57,58,59,65,67,71,72,75,76,77,78,79,81,27,32,35,43,45,48,53,54,58,59,60,61,62,63,64,67,71,72,81,27,31,35,43,46,47,51,52,53,54,58,67,71,72,81,27,30,35,47,51,54,57,67,72,81,28,31,32,34,43,47,48,49,51,54,57,62,67,72,73,74,75,80,28,33,34,43,44,47,50,51,54,56,61,62,67,72,75,79,29,36,37,38,39,40,41,42,43,44,48,51,55,56,60,63,67,72,75,79,30,31,32,33,34,36,40,44,47,50,51,55,56,59,63,68,69,70,72,75,80,30,31,34,36,39,44,45,46,49,51,52,53,54,57,58,64,69,72,75,81,30,31,34,36,39,43,44,47,48,57,58,64,68,71,75,81,29,30,31,32,33,34,36,38,42,44,46,47,48,56,59,60,61,62,63,64,67,70,71,72,73,75,81,27,28,31,32,35,39,40,41,43,45,47,48,49,50,51,55,64,65,67,69,70,74,75,80,27,32,34,36,38,39,43,44,46,47,50,52,54,62,63,66,67,68,70,76,79,28,29,30,31,33,34,35,36,37,38,39,40,41,42,43,44,45,46,51,52,53,54,55,56,57,58,59,60,61,65,67,68,70,79,29,30,31,32,35,36,41,43,45,53,64,67,68,70,79,30,31,36,40,43,44,53,64,67,68,70,79,30,31,36,37,38,39,40,41,42,43,53,54,64,67,68,70,79,29,30,31,36,39,44,53,54,64,67,68,70,78,28,30,32,36,38,39,45,46,53,54,65,67,68,70,77,27,29,30,31,32,33,34,35,36,37,38,40,47,48,49,50,51,52,53,54,66,67,68,70,76,27,29,30,31,36,39,41,42,46,54,55,56,57,58,59,60,61,62,63,64,65,66,68,69,75,27,28,30,31,32,33,35,38,40,43,45,46,49,50,51,54,55,65,68,74,27,28,29,32,33,34,35,36,37,41,42,43,44,47,48,50,53,56,64,67,68,71,72,73,27,28,30,31,35,44,46,47,49,53,57,58,60,61,62,63,64,65,66,68,71,24,25,26,27,29,32,35,43,45,48,49,52,54,55,56,57,58,59,62,68,70,23,24,25,26,29,33,34,35,44,48,49,51,53,60,61,67,70,25,26,27,29,35,36,42,43,48,49,51,52,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,25,28,36,42,48,50,51,53,56,57,58,59,60,62,63,64,65,66,67,25,26,27,28,36,37,38,41,49,52,53,54,55,56,60,61,66,67,37,39,40,41,48,49,52,53,37,40,48,49,50,51,53,37,40,48,49,53,37,39,48,49,50,51,52,37,40,48,49,37,41,42,43,37,43,36,38,39,40,41,42,36,38,35,39,35,40,41,33,34,41,32,34,35,41,31,34,36,37,38,39,41,30,32,33,39,40,41,30,31,32
};

static int DDXX[] = {30,31,31,32,32,33,33,33,33,33,33,34,34,34,34,34,34,34,35,35,35,35,35,35,36,36,36,36,37,37,37,37,37,37,37,38,38,38,38,38,39,39,39,39,39,40,40,40,40,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,42,42,42,42,42,42,42,42,43,43,43,43,43,43,43,43,43,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,53,54,54,54,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,59,60,60,60,60,60,60,61,61,61,61,61,62,62,62,63,63,63,64,64,64,64,64,64,64,64,65,65,65,65,65,65,65,65,65,65,65,65,66,66,66,66,66,66,66,66,67,67,68,68,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,78,78,78,78,78,79,79,80,80,80,81,81,81,81,81,81,81,82,82,82,83,83,84,84,84,84,84,84,84,85,85,85,85,85,85,85,86,86,86,86,87,87,
};

static int DDXY[] = {46,45,46,44,45,44,46,49,50,51,52,44,45,47,48,50,51,53,45,46,47,49,52,53,45,48,52,53,45,46,47,48,49,51,52,44,45,50,51,52,43,44,45,50,52,45,50,51,52,45,49,50,53,54,55,56,57,58,59,62,63,64,65,69,70,71,72,45,48,57,58,59,60,61,62,66,67,68,72,73,74,75,45,47,58,59,63,66,67,76,77,45,46,59,63,64,76,45,51,52,53,54,58,60,63,65,75,76,45,50,55,58,60,64,65,76,44,46,49,55,58,63,64,65,66,76,44,47,49,56,57,63,65,67,76,44,47,48,49,50,51,56,57,62,65,67,76,45,48,50,51,52,53,54,55,56,57,58,63,65,67,68,69,70,74,75,46,47,48,49,50,58,59,60,61,62,63,64,65,68,69,74,41,42,43,46,48,49,51,56,59,60,61,62,65,66,67,70,71,72,73,75,76,77,41,43,45,46,47,50,51,52,55,57,58,67,68,72,73,74,76,77,40,41,43,44,46,47,50,51,53,54,68,69,73,76,41,42,43,48,50,52,58,59,70,71,73,75,42,43,44,49,50,52,57,59,71,72,73,75,44,45,46,47,48,49,50,51,53,56,59,70,73,74,75,45,46,50,52,53,55,58,59,69,73,75,47,48,49,50,53,54,58,69,50,53,54,56,57,69,50,51,52,55,69,51,52,69,51,52,69,51,52,66,67,68,69,70,71,52,64,65,68,69,70,71,72,73,74,75,76,52,61,62,63,66,67,68,75,52,61,50,51,52,53,54,55,56,57,58,61,50,51,53,54,55,56,57,59,61,49,50,52,53,54,55,56,57,58,60,61,41,47,48,49,52,54,57,59,60,41,42,45,46,47,48,50,51,53,54,55,56,57,58,59,60,42,43,44,49,51,52,54,55,57,59,60,42,43,49,51,52,55,58,60,42,44,49,50,51,52,55,57,58,60,42,44,49,50,52,55,56,58,60,43,44,49,50,51,55,58,59,43,44,55,58,61,58,61,58,59,61,55,56,57,58,59,60,61,54,55,58,53,59,52,53,56,57,58,60,61,51,52,54,55,58,59,60,52,53,58,59,57,58,
};

static int SMDXX[] = {17,17,17,17,17,18,18,18,18,18,18,19,19,19,19,19,19,19,19,19,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,21,21,21,21,21,21,21,21,22,22,22,22,22,22,22,23,23,23,23,23,23,23,24,24,24,24,24,24,24,25,25,25,25,25,25,25,25,26,26,26,26,26,26,26,27,27,27,27,27,27,27,27,27,27,28,28,28,28,28,28,29,29,29,29,29,29,29,29,29,29,29,29,29,29,30,30,30,30,30,30,30,30,31,31,31,31,31,31,31,31,31,31,32,32,32,32,32,32,32,33,33,33,33,33,33,33,33,34,34,34,34,34,34,35,35,35,35,35,35,36,36,36,36,36,36,36,36,36,36,36,36,37,37,37,37,37,37,37,37,37,37,37,38,38,38,38,38,38,39,39,39,39,39,39,40,40,40,40,40,40,40,41,41,41,41,41,41,41,41,42,42,42,42,42,42,42,43,43,43,43,43,43,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,48,48,48,48,48,48,48,48,49,49,49,49,49,49,49,49,50,50,50,50,50,50,51,51,51,51,51,51,52,52,52,52,52,52,52,52,52,52,52,52,52,53,53,53,53,53,53,53,54,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,59,59,59,59,59,59,59,60,60,60,60,60,61,61,61,61,62,62,62,62,62,62,62,62,63,63,63,63,63,63,63,63,63,63,64,64,64,64,64,64,64,64,64,64,64,65,65,65,65,65,65,66,66,66,66,66,66,67,67,67,67,67,68,68,68,68,68,68,68,68,69,69,69,69,69,69,69,69,70,70,70,70,70,70,70,70,70,71,71,71,71,71,71,71,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,73,73,73,73,73,73,73,73,73,73,73,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,74,75,75,75,75,75,75,75,75,75,75,75,75,76,76,76,76,76,76,76,76,76,76,76,77,77,77,77,77,77,77,77,77,77,77,77,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,79,79,79,79,79,79,79,79,79,79,79,79,79,79,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,87,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,89,89,89,89,89,89,89,89,89,89,89,90,90,90,90,90,90,90,90,90,90,91,91,91,91,91,91,91,91,91,92,92,92,92,92,92,92,92,92,93,93,93,93,93,93,93,93,94,94,94,94,94,94,94,94,94,95,95,95,95,95,95,95,95,95,95,95,95,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,97,97,97,97,97,97,97,97,97,97,97,97,97,98,98,98,98,98,98,98,98,98,98,98,98,99,99,99,99,99,99,99,99,99,99,99,99,100,100,100,100,100,100,100,100,100,100,100,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,103,103,103,103,103,103,103,103,103,104,104,104,104,104,104,104,104,105,105,105,105,105,105,105,105,106,106,106,106,106,106,106,106,106,107,107,107,107,107,107,107,107,107,107,107,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,110,110,110,110,110,110,110,110,110,110,110,110,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,113,113,113,113,113,113,113,113,113,113,113,113,113,113,113,114,114,114,114,114,114,114,114,114,115,115,115,115,115,115,115,115,115,115,115,115,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,118,118,118,118,118,118,118,118,118,118,118,118,118,119,119,119,119,119,119,119,119,119,119,119,119,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,122,122,122,122,122,122,122,122,122,122,122,122,123,123,123,123,123,123,123,123,123,124,124,124,124,124,124,124,124,124,124,124,125,125,125,125,125,125,125,125,125,125,126,126,126,126,126,126,126,126,126,126,127,127,127,127,127,127,127,127,127,127,127,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,129,129,129,129,129,129,129,129,129,129,129,130,130,130,130,130,130,130,130,130,131,131,131,131,131,131,131,132,132,132,132,133,133,133,133,133,133,133,133,133,133,134,134,135,135,135,135,135,135,135,135,135,135,135,135
};

static int SMDXY[] = {81,82,83,84,85,78,79,80,86,100,101,76,77,87,88,89,90,91,99,102,74,75,78,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,103,72,73,77,79,80,83,93,103,70,71,76,81,82,94,103,69,75,95,99,100,101,104,69,75,95,100,102,105,106,68,76,95,100,103,107,108,109,67,75,76,95,101,104,109,67,74,95,96,97,98,99,102,105,109,66,74,99,103,105,110,66,69,70,71,72,73,74,99,104,105,111,112,113,114,65,66,67,68,99,104,106,115,64,65,100,104,107,108,109,110,111,116,63,64,100,104,111,117,118,62,64,101,104,111,119,120,121,61,64,101,104,111,120,60,64,100,104,111,119,54,55,56,57,60,64,100,105,111,114,115,119,54,58,59,63,99,106,111,112,113,115,119,53,62,99,107,115,119,52,61,100,108,115,119,51,60,101,102,109,115,120,50,59,103,110,115,121,122,123,49,58,104,105,110,115,124,48,57,105,110,115,125,47,56,106,110,114,115,116,117,118,125,43,44,45,46,56,107,110,119,124,41,42,50,51,52,53,56,107,111,120,123,40,48,49,53,54,55,56,107,111,119,122,38,39,41,42,43,44,45,46,47,56,107,112,118,121,36,37,40,107,112,116,117,120,35,39,107,112,115,119,34,39,107,112,114,119,31,32,33,34,35,36,37,38,39,108,112,114,119,30,31,108,111,114,118,119,29,31,107,111,114,117,21,22,23,24,28,31,106,111,114,116,20,24,26,27,28,29,30,31,32,106,111,113,115,21,23,25,26,105,111,113,115,22,23,24,25,105,110,112,115,21,22,105,109,110,111,115,21,106,108,110,115,21,107,111,114,18,19,20,21,22,107,111,114,16,17,18,19,21,23,106,107,110,114,19,20,21,22,102,103,104,105,107,110,114,20,21,102,107,110,114,21,102,103,107,111,114,21,104,107,111,114,21,40,41,42,105,107,110,113,21,38,39,42,106,107,110,113,18,19,20,38,43,105,107,110,113,19,37,42,105,107,110,114,18,20,33,34,35,36,37,38,39,40,41,105,108,109,115,116,19,21,31,32,36,37,105,109,110,117,118,20,22,23,24,25,26,27,28,29,30,32,36,37,38,105,109,111,118,20,25,29,33,35,38,39,105,106,109,112,118,21,24,26,29,32,35,39,107,109,113,118,22,24,27,29,32,35,36,40,108,109,114,119,23,24,28,29,31,35,36,39,41,108,109,115,116,117,118,119,120,121,122,123,124,29,31,32,33,34,35,36,39,42,108,109,112,120,124,38,42,43,44,45,46,47,48,49,50,51,52,53,54,55,72,73,76,82,83,84,85,86,87,107,108,109,111,113,114,121,123,124,125,37,41,44,48,49,50,51,55,72,74,75,76,82,84,85,86,106,109,110,115,120,121,122,124,126,36,40,45,46,47,51,52,53,54,55,72,74,75,76,82,83,105,106,109,110,113,116,117,118,120,121,123,126,35,36,37,38,39,72,73,82,104,108,109,112,114,119,120,122,123,126,71,72,76,77,78,79,80,81,82,101,102,103,107,108,111,115,116,117,118,119,120,121,123,126,71,74,75,76,82,103,106,107,111,120,121,122,123,124,125,126,127,128,129,69,70,73,75,77,78,82,104,111,120,122,123,125,129,130,67,68,72,74,77,78,82,104,109,110,120,122,123,124,126,129,68,69,71,73,74,76,78,82,103,107,108,121,123,125,127,128,70,71,72,75,78,81,102,106,122,126,127,71,74,77,78,80,101,105,123,125,126,71,73,76,78,80,101,105,123,124,71,72,75,78,80,101,105,122,124,71,74,77,80,100,105,122,124,70,71,74,77,80,100,105,122,123,48,49,50,51,69,71,74,76,80,100,104,122,48,50,52,69,71,72,73,74,75,79,101,102,103,104,122,48,50,52,68,70,71,72,74,76,77,78,122,123,48,50,52,67,69,70,72,73,74,75,122,124,48,49,50,53,66,67,68,69,72,74,122,124,49,50,51,54,55,65,68,72,74,122,124,50,52,55,56,57,58,59,60,61,62,63,64,69,72,73,122,124,50,51,53,54,55,56,57,58,63,64,65,69,72,73,117,118,119,120,121,124,51,52,53,54,66,69,72,116,123,51,52,53,66,69,72,116,122,51,52,53,67,69,72,116,121,51,52,53,67,70,72,89,116,120,50,51,52,53,68,71,72,73,89,116,119,44,45,49,50,51,52,53,54,67,70,72,74,88,89,116,118,45,46,47,48,49,50,52,53,54,55,56,67,68,69,73,75,87,89,116,118,46,47,48,49,50,56,74,76,86,88,115,119,47,48,49,50,51,52,53,54,55,56,75,77,85,88,110,111,112,113,114,118,119,120,50,52,53,54,55,56,76,78,79,80,81,82,83,84,88,110,117,121,122,49,50,51,52,76,79,80,87,109,116,122,123,124,125,126,76,79,80,84,85,86,108,110,116,76,78,81,84,92,93,102,107,108,109,111,117,75,78,82,83,84,85,86,87,90,91,94,100,101,103,104,105,106,108,112,113,117,75,77,78,82,83,84,87,91,95,99,108,109,114,115,116,75,76,77,81,83,84,87,92,96,97,98,108,109,81,82,83,84,87,92,101,102,103,107,108,110,83,85,87,90,91,92,95,96,97,100,104,105,106,108,111,83,84,85,86,87,88,89,93,94,95,96,97,98,99,108,112,84,85,87,92,95,99,100,101,102,103,108,113,84,85,87,91,92,95,100,108,114,84,85,87,93,94,95,99,108,110,111,114,85,88,95,96,100,105,106,109,111,114,85,89,90,95,97,100,104,107,111,113,85,91,92,95,97,101,103,108,109,111,113,85,86,87,88,89,90,91,92,93,94,95,98,99,102,110,113,85,87,94,100,101,102,103,104,105,106,112,85,88,89,90,91,92,95,107,112,85,93,94,96,108,109,111,95,97,98,110,95,99,100,101,102,103,104,105,106,107,96,107,97,98,99,100,101,102,103,104,105,106,107,108
};

int QuBiao (int MapId)
{
	int Xy = -1;
	int biao = -1;

	if (MapId == 2000) {//渔村
		Xy = rand()%1019;
		biao = YCXX[Xy]*1000;
		biao += YCXY[Xy];
	}else if (MapId == 3000) {//加加村
		Xy = rand()%456;
		biao = JJXX[Xy]*1000;
		biao += JJXY[Xy];
	}else if (MapId == 1000) {//卡村
		Xy = rand()%790;
		biao = KLXX[Xy]*1000;
		biao += KLXY[Xy];
	}else if (MapId == 4000) {//萨村
		Xy = rand()%1109;
		biao = SCXX[Xy]*1000;
		biao += SCXY[Xy];
	}else if (MapId == 5000) {//福村
		Xy = rand()%280;
		biao = FCXX[Xy]*1000;
		biao += FCXY[Xy];
	}else if (MapId == 3400) {//奇咯咯
		Xy = rand()%272;
		biao = QGGXX[Xy]*1000;
		biao += QGGXY[Xy];
	}else if (MapId == 5100) {//达那
		Xy = rand()%201;
		biao = DNXX[Xy]*1000;
		biao += DNXY[Xy];
	}else if (MapId == 1100) {//可奥村
		Xy = rand()%1263;
		biao = KAXX[Xy]*1000;
		biao += KAXY[Xy];
	}else if (MapId == 3100) {//塔姆村
		Xy = rand()%783;
		biao = TMXX[Xy]*1000;
		biao += TMXY[Xy];
	}else if (MapId == 1400) {//卡坦村
		Xy = rand()%697;
		biao = KTXX[Xy]*1000;
		biao += KTXY[Xy];
	}else if (MapId == 3300) {//乌鲁力村
		Xy = rand()%401;
		biao = WLXX[Xy]*1000;
		biao += WLXY[Xy];
	}else if (MapId == 1300) {//霍尔特村
		Xy = rand()%1016;
		biao = HETXX[Xy]*1000;
		biao += HETXY[Xy];
	}else if (MapId == 3200) {//多多村
		Xy = rand()%484;
		biao = DDXX[Xy]*1000;
		biao += DDXY[Xy];
	}else if (MapId == 400) {//沙姆岛
		Xy = rand()%1330;
		biao = SMDXX[Xy]*1000;
		biao += SMDXY[Xy];
	}

	return biao;
}
#endif
